
#ifndef ROOT_TTreeRow
#define ROOT_TTreeRow

//+SEQ,CopyRight,T=NOINCLUDE.

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TTreeRow                                                             //
//                                                                      //
// Class defining interface to a row of a TTree query result.           //
// Objects of this class are created by TTreeResult methods.            //
//                                                                      //
// Related classes are TTreeResult.                                     //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TSQLRow
//*KEEP,TSQLRow,T=C++.
#include "TSQLRow.h"
//*KEND.
#endif

class TString;


class TTreeRow : public TSQLRow {

friend class TTreeResult;

private:
   Int_t        fColumnCount;  // number of columns in row
   TString     *fFields;       // array containing result strings
   TTreeRow    *fOriginal;     // pointer to original row

   Bool_t  IsValid(Int_t field);
   TTreeRow(TSQLRow *original);

public:
   TTreeRow(Int_t nfields);
   virtual ~TTreeRow();

   void        AddField(Int_t field, const char *fieldvalue);

   void        Close(Option_t *option="");
   ULong_t     GetFieldLength(Int_t field);
   const char *GetField(Int_t field);

   ClassDef(TTreeRow,0)  // One row of an TTree query result
};

#endif
